"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSelectionError = exports.MessageResolutionError = exports.MessageDataModelError = exports.MissingCharError = exports.MessageSyntaxError = exports.MessageError = void 0;
class MessageError extends Error {
    constructor(type, message) {
        super(message);
        this.type = type;
    }
}
exports.MessageError = MessageError;
class MessageSyntaxError extends MessageError {
    constructor(type, start, end, char) {
        let message;
        switch (type) {
            case 'empty-token':
            case 'bad-escape':
            case 'bad-local-var':
            case 'bad-selector':
            case 'extra-content':
            case 'parse-error':
                message = `Syntax parse error: ${type} at ${start}`;
                break;
            case 'missing-char':
                message = `Syntax parse error: Missing character ${char} at ${start}`;
                break;
            case 'key-mismatch':
            case 'missing-fallback':
                message = `Data model error: ${type}`;
                if (start >= 0)
                    message += ` at ${start}`;
        }
        super(type, message);
        this.start = start;
        this.end = end;
    }
}
exports.MessageSyntaxError = MessageSyntaxError;
class MissingCharError extends MessageSyntaxError {
    constructor(pos, char) {
        super('missing-char', pos, pos + 1, char);
        this.char = char;
    }
}
exports.MissingCharError = MissingCharError;
class MessageDataModelError extends MessageSyntaxError {
}
exports.MessageDataModelError = MessageDataModelError;
class MessageResolutionError extends MessageError {
    constructor(type, message, source) {
        super(type, message);
        this.source = source;
    }
}
exports.MessageResolutionError = MessageResolutionError;
class MessageSelectionError extends MessageError {
    constructor(type) {
        super(type, `Selection error: ${type}`);
    }
}
exports.MessageSelectionError = MessageSelectionError;
