"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = void 0;
const pattern_1 = require("../pattern");
function validateParts(parts, functions) {
    for (const part of parts) {
        if ((0, pattern_1.isExpression)(part) &&
            (0, pattern_1.isFunctionRef)(part.body) &&
            part.body.kind === 'value') {
            if (typeof functions[part.body.name] !== 'function') {
                throw new ReferenceError(`Runtime function not available: ${part.body.name}`);
            }
            // TODO: Once runtime arg requirements are defined, test against them
        }
    }
}
/**
 * Validate a message.
 *
 * @remarks
 * Throws if `msg` is not a pattern or select message,
 * or if it references runtime functions that are not available in the `functions`.
 *
 * Formatting a message that passes validation may still fail,
 * as it may depend on parameters that are not passed in,
 * or its runtime function calls may fail.
 *
 * @beta
 */
function validate(msg, functions) {
    switch (msg.type) {
        case 'message':
            validateParts(msg.pattern.body, functions);
            break;
        case 'select':
            validateParts(msg.selectors, functions);
            for (const { value } of msg.variants) {
                validateParts(value.body, functions);
            }
            break;
        default:
            // @ts-expect-error With TS, this condition should never be reached.
            throw new Error(`Invalid message type: ${msg.type}`);
    }
}
exports.validate = validate;
