import { Message } from './data-model';
import { MessageFunctions, MessagePart } from './runtime';
/** @beta */
export interface MessageFormatOptions {
    /**
     * If given multiple locales,
     * determines which algorithm to use when selecting between them;
     * the default for `Intl` formatters is `'best fit'`.
     *
     * @remarks
     * See: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#locale_negotiation
     */
    localeMatcher?: 'best fit' | 'lookup';
    /**
     * The set of custom functions available during message resolution.
     * Extends {@link defaultFunctions}.
     */
    functions?: MessageFunctions;
}
/**
 * Create a new message formatter.
 *
 * @beta
 */
export declare class MessageFormat {
    #private;
    constructor(source: string | Message, locales?: string | string[], options?: MessageFormatOptions);
    format(msgParams?: Record<string, unknown>, onError?: (error: unknown) => void): string;
    formatToParts(msgParams?: Record<string, unknown>, onError?: (error: unknown) => void): MessagePart[];
    resolvedOptions(): {
        functions: Readonly<Readonly<MessageFunctions>>;
        localeMatcher: "lookup" | "best fit";
        locales: string[];
        message: Readonly<Message>;
    };
    private createContext;
}
