import type { Context } from '../format-context.js';
import type { Literal } from './literal.js';
import type { VariableRef } from './variable-ref.js';
/**
 * To resolve a FunctionRef, an externally defined function is called.
 *
 * @remarks
 * The `name` identifies a function that takes in the arguments `args`, the
 * current locale, as well as any `options`, and returns some corresponding
 * output. Likely functions available by default would include `'plural'` for
 * determining the plural category of a numeric value, as well as `'number'`
 * and `'date'` for formatting values.
 *
 * @beta
 */
export interface FunctionRef {
    type: 'function';
    kind: 'open' | 'close' | 'value';
    name: string;
    operand?: Literal | VariableRef;
    options?: Option[];
}
/**
 * {@link FunctionRef} options are expressed as
 * `key`/`value` pairs to allow their order to be maintained.
 *
 * @beta
 */
export interface Option {
    name: string;
    value: Literal | VariableRef;
}
/**
 * Type guard for {@link FunctionRef} pattern elements
 *
 * @beta
 */
export declare const isFunctionRef: (part: any) => part is FunctionRef;
export declare function functionRefSource(kind: FunctionRef['kind'], name: string): string;
export declare function resolveFunctionRef(ctx: Context, { kind, operand, name, options }: FunctionRef): import("../runtime/index.js").MessageValue;
