"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveFunctionRef = exports.functionRefSource = exports.isFunctionRef = void 0;
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../runtime/index.js");
const value_js_1 = require("./value.js");
/**
 * Type guard for {@link FunctionRef} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isFunctionRef = (part) => !!part && typeof part === 'object' && part.type === 'function';
exports.isFunctionRef = isFunctionRef;
function functionRefSource(kind, name) {
    switch (kind) {
        case 'open':
            return `+${name}`;
        case 'close':
            return `-${name}`;
        default:
            return `:${name}`;
    }
}
exports.functionRefSource = functionRefSource;
function resolveFunctionRef(ctx, { kind, operand, name, options }) {
    let source;
    try {
        let fnInput;
        if (operand) {
            fnInput = [(0, value_js_1.resolveValue)(ctx, operand)];
            source = (0, value_js_1.getValueSource)(operand);
        }
        else {
            fnInput = [];
        }
        source ?? (source = functionRefSource(kind, name));
        switch (kind) {
            case 'open':
            case 'close': {
                const opt = options?.length ? resolveOptions(ctx, options) : undefined;
                return (0, index_js_1.markup)(source, kind, name, opt, fnInput[0]);
            }
            default: {
                const rf = ctx.functions[name];
                if (!rf) {
                    throw new errors_js_1.MessageError('missing-func', `Unknown function ${name}`);
                }
                const msgCtx = new index_js_1.MessageFunctionContext(ctx, source);
                const opt = resolveOptions(ctx, options);
                return rf(msgCtx, opt, ...fnInput);
            }
        }
    }
    catch (error) {
        ctx.onError(error);
        source ?? (source = (0, value_js_1.getValueSource)(operand) ?? functionRefSource(kind, name));
        return (0, index_js_1.fallback)(source);
    }
}
exports.resolveFunctionRef = resolveFunctionRef;
function resolveOptions(ctx, options) {
    const opt = Object.create(null);
    if (options) {
        for (const { name, value } of options) {
            opt[name] = (0, value_js_1.resolveValue)(ctx, value);
        }
    }
    return opt;
}
