import type { Context } from '../format-context.js';
import type { Literal } from './literal.js';
import type { VariableRef } from './variable-ref.js';
/**
 * When the parser encounters an expression with syntax reserved for later use,
 * it emits a Reserved pattern element to represent it.
 *
 * @remarks
 * As the meaning of this syntax is not yet defined or supported,
 * it will always resolve with a fallback representation and emit an error.
 *
 * @beta
 */
export interface Reserved {
    type: 'reserved';
    sigil: '!' | '@' | '#' | '%' | '^' | '&' | '*' | '<' | '>' | '?' | '~';
    source: string;
    name?: never;
    operand?: Literal | VariableRef;
}
/**
 * Type guard for {@link Reserved} pattern elements
 *
 * @beta
 */
export declare const isReserved: (part: any) => part is Reserved;
export declare function resolveReserved(ctx: Context, { operand, sigil, source }: Reserved): import("../runtime/fallback.js").MessageFallback;
