import type { Context } from '../format-context.js';
import { MessageValue } from '../runtime/index.js';
import type { Expression } from './index.js';
/**
 * The value of a VariableRef is defined by the current Scope.
 *
 * @remarks
 * To refer to an inner property of an object value, use `.` as a separator;
 * in case of conflict, the longest starting substring wins.
 * For example, `'user.name'` would be first matched by an exactly matching top-level key,
 * and in case that fails, with the `'name'` property of the `'user'` object:
 * The runtime scopes `{ 'user.name': 'Kat' }` and `{ user: { name: 'Kat' } }`
 * would both resolve a `'user.name'` VariableRef as the string `'Kat'`.
 *
 * @beta
 */
export interface VariableRef {
    type: 'variable';
    name: string;
}
/**
 * Declarations aren't resolved until they're requierd,
 * and their resolution order matters for variable resolution.
 * This internal class is used to store any required data,
 * and to allow for `instanceof` detection.
 * @private
 */
export declare class UnresolvedExpression {
    expression: Expression;
    scope: Context['scope'];
    constructor(expression: Expression, scope: Context['scope']);
}
/**
 * Type guard for {@link VariableRef} pattern elements
 *
 * @beta
 */
export declare const isVariableRef: (part: any) => part is VariableRef;
export declare function lookupVariableRef(ctx: Context, { name }: VariableRef): unknown;
export declare function getMessageValue(ctx: Context, source: string, value: unknown): MessageValue | undefined;
export declare function resolveVariableRef(ctx: Context, ref: VariableRef): MessageValue;
