import { Context } from '../format-context.js';
import { datetime } from './datetime.js';
import { number } from './number.js';
import { string } from './string.js';
export { MessageDateTime, MessageDateTimePart, datetime } from './datetime.js';
export { MessageFallback, MessageFallbackPart, fallback } from './fallback.js';
export { MessageMarkup, MessageMarkupPart, markup } from './markup.js';
export { MessageNumber, MessageNumberPart, number } from './number.js';
export { MessageString, MessageStringPart, string } from './string.js';
export { MessageUnknownPart, MessageUnknownValue, unknown } from './unknown.js';
export { asBoolean, asPositiveInteger, asString, mergeLocales } from './utils.js';
export interface MessageValue {
    readonly type: string;
    readonly locale: string;
    readonly source: string;
    readonly options?: Readonly<object>;
    selectKey?: (keys: Set<string>) => string | null;
    toParts?: () => MessageExpressionPart[];
    toString?: () => string;
    valueOf?: () => unknown;
}
export interface MessageExpressionPart {
    type: string;
    source: string;
    locale?: string;
    parts?: Array<{
        type: string;
        source?: string;
        value?: unknown;
    }>;
    value?: unknown;
}
export interface MessageLiteralPart {
    type: 'literal';
    value: string;
}
export type MessagePart = MessageExpressionPart | MessageLiteralPart;
/**
 * The default Runtime includes three functions, `datetime`, `number` and `string`.
 *
 * @beta
 */
export declare const defaultFunctions: {
    datetime: typeof datetime;
    number: typeof number;
    string: typeof string;
};
export declare class MessageFunctionContext {
    #private;
    readonly source: string;
    constructor(ctx: Context, source: string);
    get localeMatcher(): "lookup" | "best fit";
    get locales(): string[];
    get onError(): (error: unknown) => void;
}
/**
 * The runtime function registry available when resolving {@link FunctionRef} elements.
 *
 * @beta
 */
export interface MessageFunctions {
    [key: string]: (context: MessageFunctionContext, options: Record<string, unknown>, input?: unknown) => MessageValue;
}
